import { ref } from "vue";
import { defineStore } from "pinia";
import type { NiagaraElecDemandData } from "../utils/types";

const useElecDemandStore = defineStore("elecDemand", () => {
  const elecData = ref<NiagaraElecDemandData[]>([]);
  const subscribers = ref<any[]>([]);

  // get data from baja
  const getElecDemandFromBaja = () => {
    // @ts-ignore
    window.require &&
      // @ts-ignore
      window.requirejs(["baja!"], (baja: any) => {
        let eleclist: NiagaraElecDemandData[] = [];
        baja.Ord.make(
          `local:|foxs:4912|station:|neql:EMS:kw|bql:select slotPath,parent.displayName,name`
        ).get({
          cursor: {
            before: () => {},
            each: (record: any) => {
              const slotPath = record.get("slotPath");
              const displayName = record.get("parent$2edisplayName");
              const name = record.get("name");

              const newItem: NiagaraElecDemandData = {
                slotPath: slotPath,
                displayName: displayName,
                name: name,
                out: 0,
              };
              eleclist.push(newItem);
            },
            after: () => {
              elecData.value = [];
              elecData.value.push(...eleclist);
              eleclist.forEach((item, index) => {
                subscribeToHistory(item, index);
              });
            },
          },
        });
      });
  };

  const subscribeToHistory = (item: NiagaraElecDemandData, index: number) => {
    const slotPath = item.slotPath;
    const ordString = `local:|foxs:4912|station:|${slotPath}`;

    // @ts-ignore
    window.require &&
      // @ts-ignore
      window.requirejs(["baja!"], (baja: any) => {
        // 建立訂閱器
        const subscriber = new baja.Subscriber();

        // 定義 changed 事件的處理函數
        subscriber.attach("changed", (prop: any) => {
          try {
            if (prop && prop.getName() === "out") {
              // 取得 out 的新值
              const match = prop.$display.match(/^(\d+(\.\d+)?)/);
              const newValue = match ? parseFloat(match[0]) : 0;
              // 更新 elecData 中對應的 out 值
              const updatedIndex = elecData.value.findIndex(
                (data) => data.slotPath === item.slotPath
              );

              if (updatedIndex !== -1) {
                const newElecData = [...elecData.value];
                newElecData[updatedIndex] = {
                  ...newElecData[updatedIndex],
                  out: Number(newValue),
                };
                elecData.value = newElecData;
                console.log(`Niagara 用電需求 ${item.name} 更新:`, newValue);
              }
            }
          } catch (error: any) {
            console.error(
              `處理 ${item.name || index} 告警變化失敗: ${error.message}`,
              error
            );
          }
        });

        baja.Ord.make(ordString)
          .get({ subscriber })
          .then(() => {
            console.log(`Successfuly subscribed to ${item.name}`);
          })
          .catch((err: any) => {
            console.error(`訂閱 ${item.name || index} 失敗: ${err.message}`);
            subscriber.detach("changed"); // 移除事件監聽器
          });
        subscribers.value.push(subscriber);
      });
  };

  const clearAllSubscriber = () => {
    subscribers.value.forEach((subscriber) => {
      subscriber.detach("changed");
      subscriber.unsubscribeAll(); // 移除所有訂閱
    });
    subscribers.value = [];
    console.log("所有訂閱已清除");
  };

  // 測試
  const getElecDatatTestBaja = () => {
    // @ts-ignore
    window.require &&
      // @ts-ignore
      window.requirejs(["baja!"], (baja: any) => {
        console.log("進入 bajaSubscriber 準備執行 BQL 訂閱");
        baja.Ord.make(
          `local:|foxs:4918|history:/testStation_MJM/MGCB_KWH?period=timerange;start=2025-04-20T00:00:00.000+08:00;end=2025-04-21T00:00:00.000+08:00|bql:history:HistoryRollup.rollup(baja:RelTime '3600000')`
        ).get({
          cursor: {
            before: () => {},
            each: (record: any) => {
              console.log("recordtest", record);
            },
            after: () => {},
            limit: -1,
            offset: 0,
          },
        });
      });
  };

  return {
    getElecDemandFromBaja,
    elecData,
    clearAllSubscriber,
    getElecDatatTestBaja,
  };
});

export default useElecDemandStore;
